APP_NAME = 'drivnal'
APP_NAME_FORMATED = 'Drivnal'
CONF_FILENAME = '%s.conf' % APP_NAME

CLOSED = 'closed'
SAVED = 'saved'
UNSAVED = 'unsaved'

FAILED = 'failed'
REMOVING = 'removing'
ABORTED = 'aborted'
ABORTING = 'aborting'
PENDING = 'pending'
WARNING = 'warning'
COMPLETE = 'complete'

CREATE_SNAPSHOT = 'create_snapshot'
REMOVE_SNAPSHOT = 'remove_snapshot'
RESTORE_OBJECT = 'restore_object'
MOVE_VOLUME = 'move_volume'

VOLUMES_UPDATED = 'volumes_updated'
SNAPSHOTS_UPDATED = 'snapshots_updated'
TASKS_UPDATED = 'tasks_updated'

SNAPSHOT_DIR = 'snapshots'
LOG_DIR = 'logs'

EVENT_DB_TTL = 61000
TASK_DB_MAX = 100

SMTP_TIMEOUT = 30
SMTP_FROM_ADDR = 'notify@drivnal.com'
SMTP_SUBJECT = 'Drivnal Notification'

MATCH_ALL = 'match_all'
MATCH_BOTH = 'match_both'
MATCH_PREFIX = 'match_prefix'
MATCH_SUFFIX = 'match_suffix'

DEFAULT_MAX_PRUNE = 0.8
DEFAULT_DB_PATH = '/var/lib/drivnal/drivnal.db'
DEFAULT_WWW_PATH = '/usr/share/drivnal/www'

DEFAULT_ROOT_EXCLUDES = [
    '/dev/*',
    '/proc/*',
    '/sys/*',
    '/tmp/*',
    '/run/*',
    '/mnt/*',
    '/media/*',
    '/lost+found',
]

DIR_MIME_TYPE = 'inode/directory'

RSYNC_WARN_EXIT_CODES = [
    23, # Partial transfer from error
    24, # Partial transfer from vanished files
]

MIN_UNITS = ['m', 'min', 'minute']
HOUR_UNITS = ['h', 'hr', 'hour']
DAY_UNITS = ['d', 'day']
WEEKDAY_UNITS = {
    'monday': 0,
    'mon': 0,
    'mo': 0,
    'm': 0,
    'tuesday': 1,
    'tues': 1,
    'tue': 1,
    'tu': 1,
    't': 1,
    'wednesday': 2,
    'wed': 2,
    'we': 2,
    'w': 2,
    'thursday': 3,
    'thur': 3,
    'thu': 3,
    'th': 3,
    'friday': 4,
    'fri': 4,
    'fr': 4,
    'f': 4,
    'saturday': 5,
    'sat': 5,
    'sa': 5,
    's': 5,
    'sunday': 6,
    'sun': 6,
    'su': 6,
}

EXTENSION_TYPES = {
    'scpt': 'applescript',
    'applescript': 'applescript',
    'as': 'actionscript3',
    'sh': 'shell',
    'cfm': 'coldfusion',
    'cfml': 'coldfusion',
    'cfc': 'coldfusion',
    'h': 'cpp',
    'hh': 'cpp',
    'hpp': 'cpp',
    'hxx': 'cpp',
    'h++': 'cpp',
    'c': 'c',
    'cc': 'cpp',
    'cpp': 'cpp',
    'cxx': 'cpp',
    'c++': 'cpp',
    'cs': 'csharp',
    'css': 'css',
    'less': 'css',
    'dfm': 'delphi',
    'delphi': 'delphi',
    'pp': 'pascal',
    'pas': 'pascal',
    'pascal': 'pascal',
    'diff': 'diff',
    'patch': 'patch',
    'erl': 'erlang',
    'hrl': 'erlang',
    'gy': 'groovy',
    'gvy': 'groovy',
    'gsh': 'groovy',
    'groovy': 'groovy',
    'hx': 'haxe',
    'hxml': 'haxe',
    'java': 'java',
    'class': 'java',
    'jfx': 'javafx',
    'javafx': 'javafx',
    'js': 'javascript',
    'json': 'javascript',
    'pl': 'perl',
    'pm': 'perl',
    't': 'perl',
    'pod': 'perl',
    'php': 'php',
    'phps': 'php',
    'php3': 'php',
    'php4': 'php',
    'php5': 'php',
    'phtml': 'php',
    'ps': 'powershell',
    'ps1': 'powershell',
    'py': 'python',
    'rb': 'ruby',
    'rbw': 'ruby',
    'sass': 'sass',
    'scss': 'scss',
    'scala': 'scala',
    'sql': 'sql',
    'svg': 'xml',
    'vb': 'vb',
    'ts': 'ts',
    'txt': 'plain',
    'xml': 'xml',
    'html': 'xml',
    'xhtml': 'xml',
    'xslt': 'xml',
    'md': 'plain',
};

# Mime match string, syntax type
MIME_TYPES = [
    ('actionscript', 'actionscript3'),
    ('applescript', 'applescript'),
    ('typescript', 'typescript'),
    ('powershell', 'powershell'),
    ('coldfusion', 'coldfusion'),
    ('plain', 'plain'),
    ('patch', 'patch'),
    ('xhtml', 'xhtml'),
    ('shell', 'shell'),
    ('text', 'plain'),
    ('bash', 'bash'),
    ('html', 'html'),
    ('diff', 'diff'),
    ('xml', 'xml'),
];

# Filename match string, syntax type
FILENAME_TYPES = [
    ('vagrantfile', 'ruby', MATCH_ALL),
    ('pkgbuild', 'bash', MATCH_ALL),
    ('bash', 'bash', MATCH_ALL),
    ('xml', 'xml', MATCH_ALL),
    ('rc', 'bash', MATCH_SUFFIX),
];

VOLUME_NOT_FOUND = 'volume_not_found'
VOLUME_NOT_FOUND_MSG = 'Volume not found'

SNAPSHOT_NOT_FOUND = 'snapshot_not_found'
SNAPSHOT_NOT_FOUND_MSG = 'Snapshot not found'

PATH_NOT_FOUND = 'path_not_found'

FILE_NOT_FOUND = 'file_not_found'
FILE_NOT_FOUND_MSG = 'File not found'



from constants import *
import os
import logging

logger = logging.getLogger(APP_NAME)
_RESERVED_ATTRIBUTES = ['bool_options', 'int_options', 'float_options',
    'path_options', 'str_options', 'list_options', 'all_options']

class Config:
    bool_options = []
    int_options = []
    float_options = []
    path_options = []
    str_options = []
    list_options = []

    def __init__(self, path=None):
        self.all_options = self.bool_options + self.int_options + \
            self.float_options + self.path_options + self.str_options + \
            self.list_options
        self._conf_path = path
        self.set_state(CLOSED)

    def __setattr__(self, name, value):
        if name in _RESERVED_ATTRIBUTES:
            pass
        elif name in self.all_options:
            self.set_state(UNSAVED)
        self.__dict__[name] = value

    def __getattr__(self, name):
        if name in _RESERVED_ATTRIBUTES:
            pass
        elif name in self.all_options:
            if self.get_state() == CLOSED:
                self.load()
            if name not in self.__dict__:
                return None
        elif name not in self.__dict__:
            raise AttributeError('Config instance has no attribute %r' % name)
        return self.__dict__[name]

    def get_state(self):
        return self._state

    def set_state(self, state):
        self._state = state

    def get_path(self):
        return self._conf_path

    def set_path(self, path):
        self._conf_path = path

    def _encode_line(self, name, value):
        if name in self.bool_options:
            value = 'true' if value else 'false'
        elif name in self.list_options:
            value = ','.join(value)

        return '%s=%s\n' % (name, value)

    def _decode_bool(self, value):
        value = value.lower()
        if value in ['true', 't', 'yes', 'y']:
            return True
        elif value in ['false', 'f', 'no', 'n']:
            return False
        else:
            raise ValueError('Value is not boolean')

    def _decode_int(self, value):
        return int(value)

    def _decode_float(self, value):
        return float(value)

    def _decode_path(self, value):
        try:
            value = os.path.normpath(value)
        except AttributeError:
            logger.error('Failed to normalize path. %r' % {
                'path': value,
            })
        return value

    def _decode_list(self, value):
        return filter(None, value.split(','))

    def _decode_line(self, line):
        line_split = line.split('=')
        name = line_split[0]
        value = '='.join(line_split[1:])

        if name not in self.all_options:
            raise ValueError('Unknown option')

        if not value:
            raise ValueError('Empty option')

        if name in self.list_options:
            values = self._decode_list(value)

            decoder = None
            if name in self.int_options:
                decoder = self._decode_int
            elif name in self.float_options:
                decoder = self._decode_float
            elif name in self.bool_options:
                decoder = self._decode_bool
            elif name in self.path_options:
                decoder = self._decode_path

            if decoder:
                for i, value in enumerate(values):
                    values[i] = decoder(value)

            value = values
        elif name in self.str_options:
            pass
        elif name in self.int_options:
            value = self._decode_int(value)
        elif name in self.float_options:
            value = self._decode_float(value)
        elif name in self.bool_options:
            value = self._decode_bool(value)
        elif name in self.path_options:
            value = self._decode_path(value)

        return name, value

    def load(self):
        logger.debug('Loading config.')
        self.set_state(SAVED)

        with open(self._conf_path) as config:
            for line in config:
                line = line.rstrip('\n')

                if line.strip() == '':
                    continue
                elif line[0] == '#':
                    continue
                elif '=' in line:
                    pass
                else:
                    logger.warning('Ignoring invalid line. %r' % {
                        'line': line,
                    })
                    continue

                try:
                    name, value = self._decode_line(line)
                    self.__dict__[name] = value
                except ValueError:
                    logger.warning('Ignoring invalid line. %r' % {
                        'line': line,
                    })

    def commit(self):
        logger.debug('Committing config.')

        with open(self._conf_path, 'w') as config:
            for name in self.all_options:
                if name not in self.__dict__:
                    continue
                value = self.__dict__[name]
                if value is None:
                    continue
                config.write(self._encode_line(name, value))

        self.set_state(SAVED)



from drivnal.constants import *
from drivnal.client import Client
from drivnal.event import Event
import drivnal.utils as utils
from drivnal import server
import time
import uuid

@server.app.route('/event', methods=['GET'])
@server.app.route('/event/<int:last_event>', methods=['GET'])
@server.app.route('/event/<volume_id>', methods=['GET'])
@server.app.route('/event/<volume_id>/<int:last_event>', methods=['GET'])
def event_get(volume_id=None, last_event=None):
    client = Client()
    volume = client.get_volume(volume_id)

    if not last_event:
        events = [
            {
                'id': uuid.uuid4().hex,
                'type': 'time',
                'time': int(time.time() * 1000),
            },
        ]
        return utils.jsonify(events)

    for i in xrange(int(10 / 0.5)):
        events = []

        for event in Event.get_events(volume, last_event):
            events.append({
                'id': event.id,
                'type': event.type,
                'time': event.time,
                'volume_id': event.volume_id,
            })

        if len(events):
            return utils.jsonify(events)

        time.sleep(0.5)

    return utils.jsonify([])



from constants import *
from core_snapshot import CoreSnapshot
import logging

logger = logging.getLogger(APP_NAME)

class RemoteSnapshot(CoreSnapshot):
    def _get_path(self):
        return ''

    def _get_log_path(self):
        return ''

    def setup_snapshot(self):
        pass

    def setup_hard_links(self, last_snapshot):
        pass

    def set_state(self, state):
        if self.state == state:
            return
        self.state = state
